<?php

namespace FiloBlu\BirthdayCoupon\Model\Magento\SalesRule\Coupon;


class Massgenerator extends \Magento\SalesRule\Model\Coupon\Massgenerator
{
    /**
     * Maximum probability of guessing the coupon on the first attempt
     */
    const MAX_PROBABILITY_OF_GUESSING = 0.25;

    /**
     * Number of attempts to generate
     */
    const MAX_GENERATE_ATTEMPTS = 10;

    /**
     * Count of generated Coupons
     * @var int
     */
    protected $generatedCount = 0;

    /**
     * @var array
     */
    protected $generatedCodes = [];

    /**
     * Sales rule coupon
     *
     * @var \Magento\SalesRule\Helper\Coupon
     */
    protected $salesRuleCoupon;

    /**
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    protected $date;

    /**
     * @var \Magento\SalesRule\Model\CouponFactory
     */
    protected $couponFactory;

    /**
     * @var \Magento\Framework\Stdlib\DateTime
     */
    protected $dateTime;

    protected $_birthdayHelper;
    protected $_resourceConnection;

    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\SalesRule\Helper\Coupon $salesRuleCoupon
     * @param \Magento\SalesRule\Model\CouponFactory $couponFactory
     * @param \Magento\Framework\Stdlib\DateTime\DateTime $date
     * @param \Magento\Framework\Stdlib\DateTime $dateTime
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb $resourceCollection
     * @param array $data
     */
    public function __construct(
        \FiloBlu\BirthdayCoupon\Helper\Data $birthdayHelper,
        \Magento\Framework\App\ResourceConnection $resourceConnection,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\SalesRule\Helper\Coupon $salesRuleCoupon,
        \Magento\SalesRule\Model\CouponFactory $couponFactory,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        \Magento\Framework\Stdlib\DateTime $dateTime,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->_birthdayHelper = $birthdayHelper;
        $this->_resourceConnection = $resourceConnection;
        $this->salesRuleCoupon = $salesRuleCoupon;
        $this->date = $date;
        $this->couponFactory = $couponFactory;
        $this->dateTime = $dateTime;
        parent::__construct($context, $registry, $salesRuleCoupon, $couponFactory, $date, $dateTime, $resource, $resourceCollection, $data);
    }

    /**
     * Initialize resource
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init('Magento\SalesRule\Model\ResourceModel\Coupon');
    }

    /**
     * Generate coupon code
     *
     * @return string
     */
    public function generateCode()
    {
        $format = $this->getFormat();
        if (empty($format)) {
            $format = \Magento\SalesRule\Helper\Coupon::COUPON_FORMAT_ALPHANUMERIC;
        }

        $splitChar = $this->getDelimiter();
        $charset = $this->salesRuleCoupon->getCharset($format);

        $code = '';
        $charsetSize = count($charset);
        $split = max(0, (int)$this->getDash());
        $length = max(1, (int)$this->getLength());
        for ($i = 0; $i < $length; ++$i) {
            $char = $charset[\Magento\Framework\Math\Random::getRandomNumber(0, $charsetSize - 1)];
            if (($split > 0) && (($i % $split) === 0) && ($i !== 0)) {
                $char = $splitChar . $char;
            }
            $code .= $char;
        }

        return $this->getPrefix() . $code . $this->getSuffix();
    }

    /**
     * Generate Coupons Pool
     *
     * @throws \Magento\Framework\Exception\LocalizedException
     * @return $this
     */
    public function generatePool()
    {
        $this->generatedCount = 0;
        $this->generatedCodes = [];
        $size = $this->getQty();
        $maxAttempts = $this->getMaxAttempts() ? $this->getMaxAttempts() : self::MAX_GENERATE_ATTEMPTS;
        $this->increaseLength();
        /** @var $coupon \Magento\SalesRule\Model\Coupon */
        $coupon = $this->couponFactory->create();
        $nowTimestamp = $this->dateTime->formatDate($this->date->gmtTimestamp());

        for ($i = 0; $i < $size; $i++) {
            $attempt = 0;
            do {
                if ($attempt >= $maxAttempts) {
                    throw new \Magento\Framework\Exception\LocalizedException(
                        __('We cannot create the requested Coupon Qty. Please check your settings and try again.')
                    );
                }

                if($size > 1 && $this->_birthdayHelper->isModuleEnabled() && $this->_birthdayHelper->isBirthdayRule($this->getRuleId())) {
                    $this->setData('suffix', '-'.$i);
                }

                $code = $this->generateCode();
                ++$attempt;
            } while ($this->getResource()->exists($code));

            $expirationDate = $this->getToDate();
            if ($expirationDate instanceof \DateTime) {
                $expirationDate = $expirationDate->format('Y-m-d H:i:s');
            }

            if($this->_birthdayHelper->isModuleEnabled() && $this->_birthdayHelper->isBirthdayRule($this->getRuleId())) {
                $expirationDate = date('Y-m-d H:i:s', strtotime('+ 10 years'));
            }

            $coupon->setId(null)
                ->setRuleId($this->getRuleId())
                ->setUsageLimit($this->getUsesPerCoupon())
                ->setUsagePerCustomer($this->getUsagePerCustomer())
                ->setExpirationDate($expirationDate)
                ->setCreatedAt($nowTimestamp)
                ->setType(\Magento\SalesRule\Helper\Coupon::COUPON_TYPE_SPECIFIC_AUTOGENERATED)
                ->setCode($code)
                ->save();

            $this->generatedCount += 1;
            $this->generatedCodes[] = $code;
        }

        if($this->_birthdayHelper->isModuleEnabled() && $this->_birthdayHelper->isBirthdayRule($this->getRuleId())) {
            $this->updateCouponExsistingSubscriber($this->getRuleId());
        }

        return $this;
    }

    public function updateCouponExsistingSubscriber($rule_id) {
        if((int)$rule_id > 0) {
            $connection = $this->_resourceConnection->getConnection();
            $tableName_coupon = $this->_resourceConnection->getTableName('salesrule_coupon');
            $tableName_subsciber_info = $this->_resourceConnection->getTableName('filoblu_newsletter_subscriber_info');
            $tableName_subsciber = $this->_resourceConnection->getTableName('newsletter_subscriber');
            $query = "SELECT * FROM ".$tableName_coupon." WHERE rule_id = ".$rule_id;
            $result = $connection->fetchAll($query);
            if($result) {
                foreach($result as $idx => $coupon) {
                    $code = $coupon['code'];
                    $split_code = explode("-", $code);
                    $coupon_day = $split_code[1];
                    $date_from_day = $this->_birthdayHelper->getDateFromDay($coupon_day);
                    if($date_from_day != '') {
                        $birthday_date = explode("-", $date_from_day);
                        $birthday_month = $birthday_date[1];
                        $birthday_day = $birthday_date[2];

                        $query_subscribers = "SELECT subscriber_id FROM ".$tableName_subsciber_info." WHERE MONTH(dob) = ".$birthday_month." AND DAY(dob) = ".$birthday_day." ORDER BY subscriber_id";
                        $subscribers = $connection->fetchAssoc($query_subscribers);
                        if($subscribers) {
                            $update_query = "UPDATE ".$tableName_subsciber." SET birthday_coupon = '".$code."' WHERE subscriber_id IN(".implode(',', array_keys($subscribers)).")";
                            $connection->query($update_query);
                        }
                    }
                }
            }
        }
    }
}
	
	