<?php
namespace FiloBlu\BirthdayCoupon\Observer;

class SetBirthdayCoupon implements \Magento\Framework\Event\ObserverInterface
{
    protected $_customer;
    protected $_birthdayHelper;

    public function __construct(
        \Magento\Customer\Model\Customer $customer,
        \FiloBlu\BirthdayCoupon\Helper\Data $birthdayHelper
    ) {
        $this->_customer = $customer;
        $this->_birthdayHelper = $birthdayHelper;
    }

  public function execute(\Magento\Framework\Event\Observer $observer)
  {
     if($this->_birthdayHelper->getConfig('filoblu_birthdaycoupon_section/options/is_enabled') == 1) {
         $subscriber = $observer->getSubscriber();
         if($subscriber) {
             $customerId = $subscriber->getData('customer_id');
             if((int)$customerId > 0) {
                 $customer = $this->_customer->load($customerId);
                 $dob = $customer->getDob();
             }
             else {
                 $dob = $subscriber->getData('dob');
             }

             if($dob) {
                 $day_of_year = $this->_birthdayHelper->getDayOfYear($dob);
                 $coupon_code = $this->_birthdayHelper->getBirthdayCouponByDay($day_of_year);
                 if($coupon_code && $coupon_code != '') {
                     $observer->getDataObject()->setData('birthday_coupon', $coupon_code);
                 }
             }
         }
     }

     return $observer;
  }
}